/* PL - ploticus using the page layout paradigm.
        For ploticus using the scrolling paginated list paradigm, use pltab.   
*/

#include "pl.h"

vermessage()
{
fprintf( Diagfp, "\n\
Usage: pl scriptfile -eps|-ps|-png|-gif|-x11 [options]\n\
\n\
ploticus 1.34 data display software (c) 1998-2000 Stephen C. Grubb\n\
Home page is www.sgpr.net.\n" );
exit( 1 );
}

main( argc, argv )
int argc;
char **argv;
{
int i, j;
char buf[256];
int use_stdin;
int vardec;
int nt;
FILE *fp;
int stat;
int found;
int valused;


/* set pre-profile defaults.. */

/* default device.. */
#ifdef NOX11
Device = 'e';
#else
Device = 'x';    
#endif
strcpy( Viewer, "" );


/* read and process profile if any.. */
strcpy( Cmdlineparms, "" );
do_preliminaries();


/* set progname for err messages.. */
strcpy( Eprogname, "pl" );


/* set defaults for command line args.. */
use_stdin = 0;
Debug = 0;
strcpy( Inputfile, "" );
Winx = 100; Winy = 0;
Winw = 8.5; Winh = 11.0;
Winsizegiven = 0;
Bkcolorgiven = 0;
strcpy( Cmdline_outfile, "" );
strcpy( Cmdlineparms, "" );
strcpy( Flags, "" );

strcpy( Flags, "I" ); /* make -id the default */

/* process command line arguments.. */
for( i = 1; i < argc; i++ ) {

	if( strcmp( argv[i], "-stdin" )==0) use_stdin = 1;

	else if( strcmp( argv[i], "-png" )==0 && !devavail( "png" ) ) { 
		char *p[100];
		if( strcmp( &argv[0][strlen(argv[0])-5], "plpng" )==0 ) {
			Eerr( 5279, "png not available in plpng", "" );
			exit(1);
			}
		p[0] = "plpng";
		for( i = 1; i < argc; i++ ) p[i] = argv[i];
		p[argc] = NULL;
		execvp( "plpng", argv );
		fprintf( Errfp, "PNG not supported in this build (plpng not found).\n" );
                exit(1);
		}

	else if( strcmp( argv[i], "-showargs" )==0 ) {
		int ii;
		fprintf( Diagfp, "-------------\n" );
		for( ii = 0; ii < argc; ii++ ) fprintf( Diagfp, "%s ", argv[ii] );
		fprintf( Diagfp, "\n" );
		}

	else 	{
		if( i+1 < argc ) process_arg( argv[i], argv[i+1], &valused, &found );
		else process_arg( argv[i], "", &valused, &found );
		if( !found && argv[i][0] == '-' ) 
			Eerr( 4892, "warning, unrecognized argument", argv[i] );
		else if( !found ) strcpy( Inputfile, argv[i] ); 
		i += valused;
		}
	}


/* copy stdin to a tmp file.. */
if( use_stdin ) {
	sprintf( Inputfile, "%s._ctrl", Tmpname );
	fp = fopen( Inputfile, "w" );
	if( fp == NULL ) {
		Eerr( 102, "Cannot open tmp file to handle stdin\n", Inputfile );
		exit(1);
		}
	while( fgets( buf, 255, stdin ) != NULL ) fprintf( fp, "%s", buf );
	fclose( fp );
	}
	
if( strlen( Inputfile ) < 1 ) {
	Eerr( 20, "No script file specified on command line", "" );
	vermessage();
	}


/* set up.. */
Esetsize( Winw, Winh, Winx, Winy );

/* DEVICE variable.. */
if( getvar( "DEVICE", buf ) != 0 ) { /* DEVICE not given on command line, set DEVICE */
	declarevar( "DEVICE" );
	devnamemap( &Device, buf, 2 );
	setvar( "DEVICE", buf );
	}
else	{ /* DEVICE given on command line, set Device from DEVICE */
	getvar( "DEVICE", buf );
	devnamemap( &Device, buf, 1 );
	}

/* set default output file names for gif and eps.. */
if( strlen( Cmdline_outfile ) < 1 ) {
    	if( Emember( Device, "eg" ) ) makeoutfilename( Inputfile, Cmdline_outfile, Device );
	}

/* if a viewcommand is specified and an outfile has not been and the device 
   is paginated postscript, we need to set the output file to out.ps */
if( strlen( Viewer ) > 0 && strlen( Cmdline_outfile ) < 1 ) {
    	if( Device == 'c' || Device == 'p' ) strcpy( Cmdline_outfile, "out.ps" );
	}


if( strlen( Cmdline_outfile ) > 0 && 
    strcmp( Cmdline_outfile, "-" )!= 0 ) 
	Esetoutfilename( Cmdline_outfile );


/* pl initializations.. */
EDXlo = 0.0; EDXhi = 0.0; EDYlo = 0.0; EDYhi = 0.0;
Initialized = 0;


/* EXECUTE THE CONTROL FILE to produce the plot.. */

stat = exec_plfile( Inputfile );
if( stat != 0 ) exit( stat );

do_logo();

/* finish up (x11: button, etc.) */
if( Initialized && Device == 'x' ) do_x_button( "End." );


if( Initialized ) {
	Eendoffile();
	}

if( !Debug ) {
	sprintf( buf, "rm %s* 2> /dev/null", Tmpname );  /* PORT */
	system( buf );
	}

if( strlen( Viewer )  > 0 && Device != 'x' ) {
	Egetoutfilename( buf );
	if( stricmp( buf, "stdout" )==0 ) fprintf( Diagfp, "Cannot use -o stdout with -viewer\n" );
	else if( strlen( buf ) > 0 ) {
		strcat( Viewer, " " );
		strcat( Viewer, buf );
		fprintf( Diagfp, "Executing '%s' to view results..\n", Viewer );
		system( Viewer );
		}
	}

exit( 0 );
}

/* --------------------- */
pltab_row( s, ypos )
char *s;
double *ypos;
{
*ypos = 0.0;
return( Eerr( 2889, "pltab scaletype not available from pl program", "" ) );
}
/* ---------------------- */
pltab_plotarea( x1, y1, x2, y2, ymax )
double *x1, *y1, *x2, *y2, *ymax;
{
return( Eerr( 2942, "pltab scaletype not available from pl program", "" ) );
}
/* ---------------------- */
pltab_setcatslide( amount )
double amount;
{ return( 0 ); }
/* ---------------------- */
pltab_nextstub( stub, maxlen)
char *stub;
int maxlen;
{ return( 0 ); }

/* ---------------------- */
pltab_in_effect()
{
return( 0 );
}
