/* RANGEBAR - boxplots */
#include "pl.x"

static int calculate_stats();
#ifdef NONANSI
static int dblcompare();
#else
static int dblcompare(double *f, double *g);
#endif

rangebar( specfile, clonefiles, nclonefiles  )
char *specfile, clonefiles[][ MAXPATH ];
int nclonefiles;
{
int i, j;
FILE *fp;
char buf[256];
char attr[40], val[80];
char lineval[256];
int nt;
FILE *openclonefile();
int icf;

int stat;
int align;
double adjx, adjy;

char baseax, axis;
int datafield;
int showstats;
double stats[20];
char values[120];
char tailmode[120];
char nlocation[40];
double nloc;
int printn;
char missword[40];
char nword[40];
char mlocation[40];
double mloc;
int printm;
double barloc;
double barwidth, hb;
double h[5];
char taildet[80];
char outlinedet[80];
char barcolor[40];
int baroutline;
char textdetails[80];
char msym[80];
double r;
double ticlen;
char selectex[120];
int nplotfields;
int pf[6];
int plotrecord;
int fnftics;
int quitaftershowingstats;
char sbarloc[20];
int meanmode;
int nstddevs;
int trunc;
double fval;
int showoutliers;
double nfcutoff, outnfcutoff;
char outnearsym[100], outfarsym[100];
int outlblfld;
char outlbldet[100];
char meansym[100];
int printoutliers;
char buf2[80];
double outlinelen;
char outlierlinedet[100];
int result;


strcpy( Eprogname, "pl proc rangebar" );

/* -------------------------- */
/* DEFAULTS */
/* All variables must start out with default values, */
/* since no variable is guaranteed to be set */
/* -------------------------- */
datafield = -1;
axis = 'y';
showstats = 0;
strcpy( values, "" );
strcpy( tailmode, "5/95" );
strcpy( nlocation, "" );
printn = 1;
strcpy( mlocation, "" );
printm = 0;
barloc = -1.0;
barwidth = 0.2;
baroutline = 1;
strcpy( barcolor, "gray(0.8)" );
strcpy( nword, "N=@N" );
strcpy( missword, "M=@M" );
strcpy( textdetails, "" );
strcpy( msym, "line" );
strcpy( taildet, "" );
strcpy( outlinedet, "" );
ticlen = -1;
strcpy( selectex, "" );
nplotfields = 0;
plotrecord = -1;
fnftics = 1;
quitaftershowingstats = 0;
strcpy( sbarloc, "" );
meanmode = 0;
nstddevs = 1;
trunc = 1;
showoutliers = 0;
strcpy( outfarsym, "shape=circle style=spokes radius=0.05" );
strcpy( outnearsym, "shape=circle style=outline radius=0.05" );
outlblfld = 0;
strcpy( outlbldet, "" );
outnfcutoff = 3.0;
strcpy( meansym, "" );
printoutliers = 0;
outlinelen = 0.0;
strcpy( outlierlinedet, "" );


/* -------------------------- */
/* open and process all clonefiles and lastly, specfile */
/* -------------------------- */
for( icf = 0; icf <= nclonefiles; icf++ ) {
 
    if( icf == nclonefiles ) {
        fp = fopen( specfile, "r" );
        if( fp == NULL ) return( Eerr( 117, "Could not open to read", specfile ) );
        }
    else        {
        fp = openclonefile( clonefiles[icf] );
        if( fp == NULL ) continue;
        }

    while( fgets( buf, 255, fp ) != NULL ) {
	nt = sscanf( buf, "%s %s", attr, val );		/* val has 2nd token */
	if( nt < 1 ) continue;
	if( nt == 1 ) strcpy( val, "" );
	if( icf < nclonefiles && stricmp( attr, "#proc" )==0 ) continue;
	i = 0;
        Egetok( buf, &i );
	while( buf[i] == ' ' || buf[i] == '\t' ) i++; /* skip over white space */
        strcpy( lineval, &buf[ i ] ); /* lineval has remainder of line, for sscanf-ing */
        lineval[ strlen( lineval ) - 1 ] = '\0'; /* strip off newline */
 
	/* strip off trailing colon if any.. */
	if( attr[ strlen( attr ) - 1 ] == ':' ) attr[ strlen( attr ) - 1 ] = '\0';

	/* attributes.. */
	if( stricmp( attr, "datafield" )==0 ) datafield = fref( val ) - 1;
	else if( stricmp( attr, "axis" )==0 ) axis = tolower(val[0]);
	else if( stricmp( attr, "barloc" )==0 ) strcpy( sbarloc, lineval ); /* val ->lineval scg */
	else if( stricmp( attr, "barwidth" )==0 ) {
		stat = num( val, &barwidth );
		if( stat != 0 ) { Eerr( 237, "barwidth invalid", val ); barwidth = 0.2; }
		if( Using_cm ) barwidth /= 2.54;
		}
	else if( stricmp( attr, "showstats" )==0 ) {
		if( strnicmp( val, YESANS, 1 )==0 ) showstats = 1;
		else if( stricmp( val, "only" )==0 ) {showstats = 1; quitaftershowingstats = 1; }
		else showstats = 0;
		}

	else if( stricmp( attr, "values" )==0 ) strcpy( values, lineval ); /* plot values given
							here for one bar
							(calculations already done exteranally) */

	else if( stricmp( attr, "plotfields" )==0 ) { /* read plot values from data array
							(calculations already done exteranally) */
		nplotfields = sscanf( lineval, "%d %d %d %d %d %d", 
			&pf[0], &pf[1], &pf[2], &pf[3], &pf[4], &pf[5], &pf[6] );
		if( nplotfields > 0 && nplotfields != 5 && nplotfields != 6 ) 
			return( Eerr( 729, 
		"plotfields: 5 or 6 datafields expected: 5thpercentile 25th median 75th 95th [N]", "" ) );
		}
	else if( stricmp( attr, "plotrecord" )==0 ) plotrecord = atoi( val ) -1; /* select which
										data record
										to plot */

	else if( stricmp( attr, "tailmode" )==0 ) strcpy( tailmode, val );
	else if( stricmp( attr, "95tics" )==0 ) {
		if( strnicmp( val, YESANS, 1 )==0 ) fnftics = 1;
		else fnftics = 0;
		}

	else if( stricmp( attr, "taildetails" )==0 ) strcpy( taildet, lineval );
	else if( stricmp( attr, "outlinedetails" )==0 ) {
		strcpy( outlinedet, lineval );
		baroutline = 1;
		}
	else if( stricmp( attr, "color" )==0 ) strcpy( barcolor, lineval );
	else if( stricmp( attr, "outline" )==0 ) {
		if( strnicmp( val, YESANS, 1 )==0 ) baroutline = 1;
		else baroutline = 0;
		}

	else if( stricmp( attr, "printn" )==0 ) {
		if( strnicmp( val, YESANS, 1 )==0 ) printn = 1;
		else printn = 0;
		}
	else if( stricmp( attr, "nlocation" )==0 ) strcpy( nlocation, val );

	else if( stricmp( attr, "printmissing" )==0 ) {
		if( strnicmp( val, YESANS, 1 )==0 ) printm = 1;
		else printm = 0;
		}
	else if( stricmp( attr, "mlocation" )==0 ) strcpy( mlocation, val );
	else if( stricmp( attr, "mediansym" )==0 ) strcpy( msym, lineval );
	else if( stricmp( attr, "mword" )==0 ) strcpy( missword, lineval );
	else if( stricmp( attr, "nword" )==0 ) strcpy( nword, lineval );
	else if( stricmp( attr, "textdetails" )==0 ) strcpy( textdetails, lineval );
	else if( stricmp( attr, "truncate" )==0 ) {
		if( strnicmp( val, YESANS, 1 )==0 ) trunc = 1;
		else trunc = 0;
		}

	else if( stricmp( attr, "ticlen" )==0 ) {
		stat = num( val, &ticlen );
		if( stat != 0 ) ticlen = -1.0;
		if( Using_cm ) barwidth /= 2.54;
		}
	else if( stricmp( attr, "select" )==0 ) strcpy( selectex, lineval );
	else if( stricmp( attr, "meanmode" )==0 ) {
		if( strnicmp( val, YESANS, 1 )==0 ) meanmode = 1;
		else meanmode = 0;
		}
	else if( stricmp( attr, "nstddevs" )==0 ) nstddevs = atoi( val );
	else if( stricmp( attr, "showoutliers" )==0 ) {
		if( strnicmp( val, YESANS, 1 )==0 ) showoutliers = 1;
		else showoutliers = 0;
		}
	else if( stricmp( attr, "outliernearfarcutoff" )==0 ) outnfcutoff = atof( val );
	else if( stricmp( attr, "outliernearsym" )==0 ) strcpy( outnearsym, lineval );
	else if( stricmp( attr, "outlierfarsym" )==0 ) strcpy( outfarsym, lineval );
	else if( stricmp( attr, "outlierlabelfield" )==0 ) outlblfld = fref( val );
	else if( stricmp( attr, "outlierlabeldetails" )==0 ) strcpy( outlbldet, lineval );
	else if( stricmp( attr, "outlierprint" )==0 ) {
		if( strnicmp( val, YESANS, 1 )==0 ) printoutliers = 1;
		else printoutliers = 0;
		}
	else if( stricmp( attr, "outlierlinelen" )==0 ) {
		outlinelen = atof( val );
		if( Using_cm ) outlinelen /= 2.54;
		}
	else if( stricmp( attr, "outlierlinedetails" )==0 ) strcpy( outlierlinedet, lineval );
	else if( stricmp( attr, "meansym" )==0 ) strcpy( meansym, lineval );
	else Eerr( 1, "attribute not recognized", attr );
	}
    fclose( fp );
    }

/* -------------------------- */
/* overrides and degenerate cases */
/* -------------------------- */
if( ( datafield < 0 || datafield >= Nfields[Dsel] ) && strlen(values) < 1 && nplotfields < 1 ) 
	return( Eerr( 601, "datafield not specified or out of range", "" ) );
if( axis == 'x' ) baseax = 'y';
else baseax = 'x';

if( !quitaftershowingstats ) {
	if( Nrecords[Dsel] < 1 && strlen( values ) < 1 ) 
		return( Eerr( 17, "No data has been read yet w/ proc getdata", "" ) );
	if( !scalebeenset() )
         return( Eerr( 51, "No scaled plotting area has been defined yet w/ proc areadef", "" ) );
	}

if( strlen( nlocation ) < 1 ) nloc = Elimit( axis, 'l', 'a' ) + 0.3;
else Eposex( nlocation, axis, &nloc );

if( strlen( mlocation ) < 1 ) mloc = Elimit( axis, 'l', 'a' ) + 0.15;
else Eposex( mlocation, axis, &mloc );

if( ticlen < 0.0 ) ticlen = barwidth * 0.7;

 

	

/* --------------------------- */
/* Do the computation work.. */
/* --------------------------- */

if( strlen( values ) > 0 ) {
	nt = sscanf( values, "%lf %lf %lf %lf %lf %lf", 
		&stats[4], &stats[5], &stats[6], &stats[7], &stats[8], &stats[0] );
	if( nt != 6 && nt != 5 ) return( Eerr( 205, 
		"values: 5 or 6 numbers expected: 5thpercentile 25th median 75th 95th [N]", "" ) );
	if( nt == 5 ) { printn = 0; stats[0] = PLHUGE; } /* no N given */
	strcpy( tailmode, "5/95" );
	printm = 0;
	}
else if( nplotfields > 0 ) {
	if( plotrecord < 0 ) return( Eerr( 2874, 
		"if plotfields are specified, plotrecord must also be specified", "" ));
	stats[4] = fda( plotrecord, pf[0]-1, axis );
	stats[5] = fda( plotrecord, pf[1]-1, axis );
	stats[6] = fda( plotrecord, pf[2]-1, axis );
	stats[7] = fda( plotrecord, pf[3]-1, axis );
	stats[8] = fda( plotrecord, pf[4]-1, axis );
	if( nplotfields == 6 ) stats[0] = fda( plotrecord, pf[5]-1, axis );
	else { printn = 0; stats[0] = PLHUGE; }
	printm = 0;
	}
else	{
	calculate_stats( datafield, axis, selectex, stats );
	if( showstats ) {
	    fprintf( Diagfp, "\n// Rangebar statistics computed on data field %d ", datafield+1 );
	    if( strlen( selectex )> 0 ) fprintf( Diagfp, "(where %s)", selectex );
	    Euprint( buf, axis, stats[1], "" );
	    fprintf( Diagfp, "\n// N=%g,  mean=%s,  stddev=%g,  #missing=%g,  sum(E)=%g\n", 
		stats[0], buf, stats[2], stats[10], stats[11] );
	    Euprint( buf, axis, stats[3], "" );
	    Euprint( buf2, axis, stats[9], "" );
	    fprintf( Diagfp, "// min=%s,  max=%s,  ", buf, buf2 );
	    Euprint( buf2, axis, stats[6], "" );
	    fprintf( Diagfp, "median=%s,  ", buf2 );
	    Euprint( buf, axis, stats[5], "" );
	    Euprint( buf2, axis, stats[7], "" );
	    fprintf( Diagfp, "25th_pctile=%s,  75th_pctile=%s,  IQR=%g\n", buf, buf2, stats[7]-stats[5] );
	    }
	}
if( quitaftershowingstats ) return( 0 );


if( !slmember( tailmode, "5* m* 1*" )) {
	Eerr( 582, "tailmode must be either 5/95, min/max, or 1.5irq", tailmode );
	strcpy( tailmode, "5/95" ); 
	}
/* -------------------------- */
/* now do the plotting work.. */
/* -------------------------- */
if( strlen( sbarloc ) < 1 ) strcpy( sbarloc, "1" );
barloc = Econv( baseax, sbarloc );
if( Econv_error() ) { 
	fprintf( Errfp, "ignoring barloc %s\n", sbarloc ); 
	return( 0 );
	}

if( baseax == 'y' ) Eflip = 1;

hb = barwidth / 2.0;
if( meanmode ) {
	h[0] = stats[1] - (stats[2] * nstddevs);
	h[2] = stats[1];
	h[4] = stats[1] + (stats[2] * nstddevs);
	}
else 	{
	if( tailmode[0] == '5' ) {
		h[0] = stats[4];
		h[4] = stats[8];
		}
	else if( tailmode[0] == '1' ) {
		h[0] = stats[6] - ((stats[7] - stats[5]) * 1.5 );
		if( h[0] < stats[3] ) h[0] = stats[3]; /* truncate to min */
		h[4] = stats[6] + ((stats[7] - stats[5]) * 1.5 );
		if( h[4] > stats[9] ) h[4] = stats[9]; /* truncate to max */
		}
	else	{
		h[0] = stats[3];
		h[4] = stats[9];
		}
	h[1] = stats[5];
	h[2] = stats[6];
	h[3] = stats[7];
	}
if( showstats ) {
	Euprint( buf, axis, h[0], "" );
	Euprint( buf2, axis, h[4], "" );
	fprintf( Diagfp, "// Tails (mode=%s): low=%s,  high=%s\n", tailmode, buf, buf2 );
	}

/* check to see if bar location is in range.. */
if( !Ef_inr( baseax, barloc ) ) {
	fprintf( Errfp, "warning, rangebar location out of %c plotting area\n", baseax );
	goto SKIPOUT;
	}

/* now convert barloc to absolute.. */
if( barloc <= 0.0 ) barloc = Ea(X,1.0);
else barloc = Ea(X,barloc);

ticlen *= 0.5;


/* if N=0, skip out */
if( stats[0] <= 0.0 ) goto SKIPOUT;

/* if entire bar is out of plotting area, skip out.. */
if( h[0] < Elimit( axis, 'l', 's' ) && h[4] < Elimit( axis, 'l', 's' ) ) {
	fprintf( Errfp, "warning, entire rangebar out of %c plotting area (under)\n", axis );
	goto SKIPOUT;
	}
if( h[0] > Elimit( axis, 'h', 's' ) && h[4] > Elimit( axis, 'h', 's' ) ) {
	fprintf( Errfp, "warning, entire rangebar out of %c plotting area (over)\n", axis );
	goto SKIPOUT;
	}

linedet( "taildetails", taildet, 1.0 );

if( !trunc || Ef_inr( axis, h[0] ) ) { /* bottom tic */
	Emov( barloc-ticlen, Ea( Y, h[0] ) ); 
	Elin( barloc+ticlen, Ea( Y, h[0] ) ); 
	}

if( !trunc || Ef_inr( axis, h[4] ) ) { /* top tic */
	Emov( barloc-ticlen, Ea( Y, h[4] ) ); 
	Elin( barloc+ticlen, Ea( Y, h[4] ) ); 
	}

/* truncate values to be within range.. */
if( trunc && !Ef_inr( axis, h[0] )) h[0] = Elimit( axis, 'l', 's' );
if( trunc && !Ef_inr( axis, h[1] )) h[1] = Elimit( axis, 'l', 's' );
if( trunc && !Ef_inr( axis, h[3] )) h[3] = Elimit( axis, 'h', 's' );
if( trunc && !Ef_inr( axis, h[4] )) h[4] = Elimit( axis, 'h', 's' );

if( meanmode ) {
	Emov( barloc, Ea( Y, h[0] ) ); 
	Elin( barloc, Ea( Y, h[4] ) ); 
	}

else	{
	Emov( barloc, Ea( Y, h[0] ) ); 
	Elin( barloc, Ea( Y, h[1] ) ); 
	Emov( barloc, Ea( Y, h[3] ) ); 
	Elin( barloc, Ea( Y, h[4] ) );
	}



if( !meanmode ) {
	linedet( "outlinedetails", outlinedet, 0.5 );
	Ecblock( barloc-hb, Ea(Y,h[1]), barloc+hb, Ea(Y,h[3]), barcolor, baroutline );
	}


/* do median */
if( strcmp( msym, "line" ) ==0 && (!trunc || Ef_inr( axis, h[2] ))) {
	Emov( barloc-hb, Ea(Y,h[2]) );
	Elin( barloc+hb, Ea(Y,h[2]) );
	}
else if( strlen( msym ) > 0 && (!trunc || Ef_inr( axis, h[2] ))) {
	symdet( "mediansym", msym, buf, &r );
	Emark( barloc, Ea(Y,h[2]), buf, r );
	}

setfloatvar( "RANGEBARMIN", h[0] );
setfloatvar( "RANGEBARMEDIAN", h[2] );
setfloatvar( "RANGEBARMAX", h[4] );
setfloatvar( "NVALUES", stats[0] );

if( tailmode[0] != '5' && tailmode[0] != '1' && fnftics && !meanmode ) { /* add 5/95 ticks to min/max tails */
	if( !trunc || Ef_inr( axis, stats[4] )) {
		Emov( barloc-(ticlen*0.8), Ea( Y, stats[4] ) );
		Elin( barloc+(ticlen*0.8), Ea( Y, stats[4] ) );
		}
	if( !trunc || Ef_inr( axis, stats[8] )) {
		Emov( barloc-(ticlen*0.8), Ea( Y, stats[8] ) );
		Elin( barloc+(ticlen*0.8), Ea( Y, stats[8] ) );
		}
	}


/* display outliers.. */
if( showoutliers ) {
	nfcutoff = (h[3] - h[1]) * outnfcutoff;
	if( Eflip ) Etextdir( 90 );
	if( strlen( outlierlinedet ) > 0 ) linedet( "outlierlinedetails", outlierlinedet, 0.5 );
	for( i = 0; i < Nrecords[Dsel]; i++ ) {
		double radius;
		char symcode[50];

		if( strlen( selectex ) > 0 ) { /* process against selection condition if any.. */
						/* added 10/29/99 */
			stat = do_select( selectex, i, &result );
			if( stat != 0 ) { Eerr( stat, "Select error", selectex ); continue; }
			if( result == 0 ) continue; /* reject */
			}

		fval = fda( i, datafield, axis );
		if( ( fval < h[0] || fval > h[4] ) && !Econv_error() ) {  /* its an outlier.. */
			if( printoutliers ) {
				fprintf( Diagfp, "// data field %d outlier: %s (", 
					datafield+1, da(i, datafield)  );
				for( j = 0; j < Nfields[Dsel]; j++ ) 
					fprintf( Diagfp, "%s ", da( i, j ) );
				fprintf( Diagfp, ")\n" );
				}
					
			if( !Ef_inr( axis, fval )) {
				fprintf( Errfp, "warning, outlier %s out of range, not displayed\n", 
					da(i,datafield) );
				continue;
				}
			if( outlinelen > 0.0 ) { /* render as lines, whether near of far */
						 /* use linedetails in effect for tails */
				Emov( barloc-(outlinelen/2.0), Ea(Y,fval) );
				Elin( barloc+(outlinelen/2.0), Ea(Y,fval) );
				}
			else if( fval < (h[2] - nfcutoff) || fval > h[2] + nfcutoff ) {    /* far */
				symdet( "outlierfarsym", outfarsym, symcode, &radius );
				Emark( barloc, Ea(Y,fval), symcode, radius );
				}
			else 	{	/* near */
				symdet( "outliernearsym", outnearsym, symcode, &radius );
				Emark( barloc, Ea(Y,fval), symcode, radius );
				}
			if( outlblfld > 0 ) {
				textdet( "outlierlabeldetails", outlbldet, &align, &adjx, &adjy, -3, "R" );
				if( align == '?' ) align = 'L';
				if( Eflip ) Emov( barloc+0.1+adjx, Ea( Y, fval )+adjy );
				else Emov( barloc+0.1+adjx, Ea( Y, fval )-((Ecurtextheight*0.35)+adjy) );
				Edotext( da( i, outlblfld-1 ), align );
				}
			}
		}
	if( Eflip ) Etextdir( 0 );
	}
		

/* display mean symbol on a median-based rangebar.. */
if( strlen( meansym ) > 0 ) {
	double radius; char symcode[50];
	if( stricmp( meansym, "yes" )==0 ) 
		strcpy( meansym, "shape=circle style=filled fillcolor=black radius=0.02" );
	symdet( "meansym", meansym, symcode, &radius );
	Emark( barloc, Ea(Y,stats[1]), symcode, radius );
	}



/* display N= and maybe M= .. */
SKIPOUT:
textdet( "textdetails", textdetails, &align, &adjx, &adjy, -2, "R" );

if( printn ) {
	sprintf( buf, "%g", stats[0] );
	Evarsub( nword, "@N", buf );
	if( baseax == 'y' ) Emov( (barloc-(Ecurtextheight*0.4)) + adjx, nloc + adjy );
	else Emov( barloc + adjx, nloc + adjy );
	Ecentext( nword );
	}
if( printm ) {
	sprintf( buf, "%g", stats[10] );
	Evarsub( missword, "@M", buf );
	Emov( barloc, mloc );
	Ecentext( missword );
	}

if( baseax == 'y' ) Eflip = 0;
return( 0 );
}


/* ========================= */
/* ========================= */
/* ========================= */
static int
calculate_stats( ifld, axis, selectex, stats )
int ifld;
char axis;
char *selectex;
double stats[];
{
double total, val, val2, mean, stddev, totsq, min, max;
double tmp;
int i, j, n, ip;
int sorted, nbad;
double sqrt();
int result;
int stat;

total = 0; n = 0, totsq=0; min = PLHUGE; max = NEGHUGE; nbad = 0;

/* read lines of data */
for( i = 0; i < Nrecords[Dsel]; i++ ) {
	
	if( strlen( selectex ) > 0 ) { /* process against selection condition if any.. */
		stat = do_select( selectex, i, &result );
		if( stat != 0 ) { Eerr( stat, "Select error", selectex ); continue; }
		if( result == 0 ) continue; /* reject */
		}
	val = fda( i, ifld, axis );
	if( Econv_error() ) {  /* will be true on bad data item */
		conv_msg( i, ifld, "datafield" );
		nbad++; 
		continue; 
		}
	if( val > max ) max = val;
	if( val < min ) min = val;
	total += val;
	totsq += val * val;
	n++;
	if( n <= MAXDAT-1 ) Dat[n] = val;
	if( n == MAXDAT-1 ) 
		Eerr( 248, "Sorry, capacity for median comp exceeded- results not valid\n" );
	}

mean = total / (double)n;
if( n > 1 ) stddev = sqrt( (totsq-( total*total /(double)n )) / (double)(n - 1)) ;
else stddev = 0.0;

/* Dat[0] will not be used */

/* bubble sort */
/*
 * while( 1 ) {
 *	sorted = 1;
 *	for( j = 1; j < n; j++ ) {	
 *		if( Dat[j] > Dat[j+1] ) {
 *			sorted = 0;
 *			tmp = Dat[j]; Dat[j] = Dat[j+1]; Dat[j+1] = tmp;
 *			}
 *		}
 *	if( sorted ) break;
 *	}
 */
qsort( &Dat[1], n, sizeof(double), dblcompare);

stats[0] = (double) n;
stats[1] = mean;
stats[2] = stddev;
stats[3] = min;
stats[4] = (n % 20 ) ? Dat[(n/20) + 1] :  (Dat[n/20] + Dat[(n/20) + 1] ) /2.0 ;  /* 5th */
stats[5] = ( n % 4 ) ?  Dat[(n/4) + 1]  :  (Dat[n/4] + Dat[(n/4) + 1])/2.0 ;      /* 25 */
stats[6] = ( n % 2 ) ?  Dat[(n+1) / 2]  :  (Dat[n/2] + Dat[(n/2)+1])/2.0 ;         /* 50 */
stats[7] = ( n % 4 )  ? Dat[n - (n/4)]  :  (Dat[(n+1) - (n/4)] + Dat[n-(n/4)])/2.0 ;   
stats[8] = ( n % 20 ) ? Dat[n - (n/20)] : (Dat[(n+1) - (n/20)] + Dat[n - (n/20)]) / 2.0 ; 
stats[9] = max;
stats[10] = (double) nbad;
stats[11] = total; 

return( 0 );
}

/* ============================= */
static int
dblcompare( f, g )
double *f, *g;
{
if( *f > *g ) return( 1 );
if( *f < *g ) return( -1 );
return( 0 );
}
