/* SCRIPTOPS - operations that are general to all script types. */

#include "dms.x"


/* =========================== */
/* DO_SET - handle a #set line */

do_set( line, data, recordid )
char *line;
char data[MAXITEMS][DATAMAXLEN+1];
char *recordid;
{
int i, ix;
char itemname[80];
int stat;
char *getok();
char tok[80];
char buf[256];
char value[256];

i = 0;
getok( line, &i ); /* get #set.. */
strcpy( itemname, getok( line, &i ) ); /* get itemname */
if( itemname[0] == '@' ) strcpy( itemname, &itemname[1] );

/* skip the optional equals sign.. */
ix = i; if( strcmp( getok( line, &ix ), "=" )==0 ) i = ix; 
while( line[i] == ' ' ) i++; /* skip over white space.. */

if( line[i] == '"' ) {  /* do value substitution only.. */
	stat =  assignvalue( itemname, &line[i+1], data, recordid );
	if( stat != 0 ) return( err( stat, 0, "Variable to be set is undeclared", line ) );
	}

/* condex 'pseudo'-functions that update multiple data items go here.. */
#ifndef PLOTICUS
else if( slmember( &line[i], "$setvector(* $setvector_virg(* $allvector(* $countvector(*" ) ) {
	char itemname1[80], itemname2[80], func[80];
	int ifld1, ifld2;
	getchunk( func, line, &i, "(" );
	i++;
	getchunk( itemname1, line, &i, "," );
	ifld1 = itemfld( recordid, itemname1 );
	if( ifld1 < 0 ) return( err( 2849, 0, "bad vector function argument", itemname1 ) );
	getchunk( itemname2, line, &i, "," );
	ifld2 = itemfld( recordid, itemname2 );
	if( ifld2 < 0 ) return( err( 2849, 0, "bad vector function argument", itemname2 ) );
	getchunk( value, line, &i, ",)" );
	if( slmember( func, "$setvector*" )) {
	    if( strcmp( func, "$setvector_virg" )==0 ) {
		/* check all fields to make sure they are virgin (= or empty or value).. */
		for( i = ifld1; i <= ifld2; i++ ) {
			if( strcmp( data[i], "=" )!=0 && strlen( data[i] ) > 0 && 
			    strcmp( data[i], value ) != 0 ) {  
				/* if not, func. returns "error" */
				assignvalue( itemname, "error", data, recordid );
				return( 0 );
				}
			}
		}
	    for( i = ifld1; i <= ifld2; i++ ) strcpy( data[i], value );
	    assignvalue( itemname, "0", data, recordid ); /* OK */
	    }

	else if( strcmp( func, "$allvector" )==0 ) {
		for( i = ifld1; i <= ifld2; i++ ) {
			if( strcmp( data[i], value ) != 0 ) {
				assignvalue( itemname, "0", data, recordid );
				return( 0 );
				}
			}
		assignvalue( itemname, "1", data, recordid );
		}
	else if( strcmp( func, "$countvector" )==0 ) {
		int count;
		count = 0;
		for( i = ifld1; i <= ifld2; i++ ) {
			if( strcmp( data[i], value ) == 0 ) count++;
			}
		sprintf( value, "%d", count );
		assignvalue( itemname, value, data, recordid );
		}
	else return( err( 2847, 0, "dont recognize", func ) );
	return( 0 );
	}

/* scripter_callf is implemented here so that data array can be passed. */
else if( slmember( &line[i], "$scripter_callf(*" )) {
	char func[80], scriptfile[80];
	struct scripterstate ss;

	getchunk( func, line, &i, "(" );
	i++;
	getchunk( scriptfile, line, &i, ")" );
	stat = scripter_open( &ss, scriptfile );
	if( stat != 0 ) return( stat );
	strcpy( value, "" );
	while( 1 ) {
        	stat = scripter( &ss, buf, recordid, data );
        	if( stat != 0 ) break;
        	strcat( value, buf );
        	if( buf[ strlen( buf ) -1 ] == '\n' ) break;
        	}
	scripter_close( &ss );
	stat = assignvalue( itemname, value, data, recordid );
	return( stat );
	}

/* $deref() do one dereference */
else if( slmember( &line[i], "$deref(*" ) ) {
	getchunk( buf, line, &i, "(" ); /* function name */
	i++;
	getchunk( value, line, &i, ")" );
	stat = value_subst( buf, value, data, recordid, NORMAL );
	if( stat > 1 ) return( stat );
	stat = assignvalue( itemname, buf, data, recordid );
	if( stat != 0 ) return( err( stat, 0, "Variable to be set is undeclared", line ) );
	}
#endif PLOTICUS
	

else	{   /* do condex evaluation */
	stat = condex( &line[i] );
	if( stat < 0 ) return( err( 402, 0, "Expression error", line ) );
	stat = assignvalue( itemname, condex_results(), data, recordid ); 
	if( stat != 0 ) return( err( stat, 0, "Variable to be set is undeclared", line ) );
	}
return( 0 );
}

/* =========================== */
/* DO_DECLARE - handle a #declare line */
/* 		data may be passed in as NULL - if so no comparison against
						#datarec item names is done. */
do_declare( line, data, recordid )
char *line;
char data[MAXITEMS][DATAMAXLEN+1];
char *recordid;
{
int i;
char itemname[80];
int stat;
char *getok();
char opt[80];

i = 0;
getok( line, &i ); /* get #declare.. */
strcpy( itemname, getok( line, &i ) ); /* get itemname */
if( data != NULL ) {
#ifndef PLOTICUS
	if( itemfld( recordid, itemname ) >= 0 ) 
		return( err( 404, 0, "#declare: data item name conflict", itemname ) );
#endif
	;
	}
strcpy( opt, getok( line, &i ) ); /* get opt */
if( strcmp( opt, "soft" )==0 ) { /* soft option.. only declare/initialize if not
					declared already */
	stat = 0;
	if( getvar( itemname, opt ) != 0 ) stat = declarevar( itemname );
	return( stat );
	}
stat = declarevar( itemname );
if( stat != 0 ) return( err( stat, 0, "#declare error", line ) );
return( 0 );
}

/* =========================== */
/* DO_MUSTHAVE - handle a #musthave line */

do_musthave( line, data, recordid )
char *line;
char data[MAXITEMS][DATAMAXLEN+1];
char *recordid;
{
int i;
char itemname[80];
int stat;
char buf[255];
char *getok();

i = 0;
getok( line, &i ); /* get #musthave.. */
strcpy( itemname, getok( line, &i ) ); /* get itemname */
stat = getvalue( buf, itemname, data, recordid );
if( stat != 0  || strlen( buf ) < 1 ) 
	return( err( 408, 0, "Required variable not set.", itemname ) );
return( 0 );
}


/* ============================= */
/* RM_COMMENTS - remove comments given to the right of statements */

rm_comments( line )
char *line;
{
int i;
char tok[100];

/* don't do this for shell commands, they may neet to contain // */
sscanf( line, "%s", tok );
if( strncmp( tok, "#shell", 6 )==0 ) return( 0 ); 

for( i = 0; i < strlen( line ); i++ ) {
	if( line[i] == '/' && line[i+1] == '/' ) {
		line[i] = '\n';
		line[i+1] = '\0';
		break;
		}
	}
return( 0 );
}
